@@ -1,90 +1,50 @@
 import re
+from copy import deepcopy
 from typing import Dict, List
-from simple_ddl_parser.parser import Parser
+
+from simple_ddl_parser import tokens as tok
 from simple_ddl_parser.dialects.hql import HQL
-
-_ref = "REFERENCES"
-_cons = "CONSTRAINT"
+from simple_ddl_parser.dialects.oracle import Oracle
+from simple_ddl_parser.parser import Parser
+from simple_ddl_parser.utils import remove_par
 
 
-class DDLParser(Parser, HQL):
+class DDLParser(Parser, HQL, Oracle):
     """
     lex and yacc parser for parse ddl into BQ schemas
     """
 
-    # statements that used at the start of defenition or in statements without columns
-    defenition_statements = {
-        "DROP": "DROP",
-        "CREATE": "CREATE",
-        "TABLE": "TABLE",
-        "ALTER": "ALTER",
-        "TYPE": "TYPE",
-        "DOMAIN": "DOMAIN",
-        "REPLACE": "REPLACE",
-        "OR": "OR"
-    }
-    common_statements = {
-        "CHECK": "CHECK",
-        "CONSTRAINT": "CONSTRAINT",
-        "FOREIGN": "FOREIGN",
-        "INDEX": "INDEX",
-        "SEQUENCE": "SEQUENCE",
-        "REFERENCES": "REFERENCES",
-        "KEY": "KEY",
-        "ADD": "ADD",
-        "AS": "AS",
-        "LIKE": "LIKE",
-        "DEFERRABLE": "DEFERRABLE",
-        "INITIALLY": "INITIALLY",
-        "IF": "IF",
-        "NOT": "NOT",
-        "EXISTS": "EXISTS",
-        "UNIQUE": "UNIQUE",
-        "ON": "ON",
-    }
-
-    columns_defenition = {
-        "DELETE": "DELETE",
-        "UPDATE": "UPDATE",
-        "NULL": "NULL",
-        "PRIMARY": "PRIMARY",
-        "DEFAULT": "DEFAULT",
-        "ARRAY": "ARRAY",
-        ",": "COMMA",
-    }
-    after_columns_tokens = {
-        "PARTITIONED": "PARTITIONED",
-        "BY": "BY",
-        "STORED": "STORED",
-        "LOCATION": "LOCATION",
-        "ROW": "ROW",
-        "FORMAT": "FORMAT",
-        "FIELDS": "FIELDS",
-        "TERMINATED": "TERMINATED",
-        "COLLECTION": "COLLECTION",
-        "ITEMS": "ITEMS",
-        "MAP": "MAP",
-        "KEYS": "KEYS",
-    }
-    sequence_reserved = {
-        "INCREMENT": "INCREMENT",
-        "START": "START",
-        "MINVALUE": "MINVALUE",
-        "MAXVALUE": "MAXVALUE",
-        "CACHE": "CACHE",
-    }
-    tokens = tuple(
-        ["ID", "DOT", "STRING", "LP", "RP", "LT", "RT", "COMMAT"]
-        + list(defenition_statements.values())
-        + list(common_statements.values())
-        + list(columns_defenition.values())
-        + list(sequence_reserved.values())
-        + list(after_columns_tokens.values())
-    )
-
+    tokens = tok.tokens
     t_ignore = ";\t  \r"
     t_DOT = r"."
 
+    def get_tag_symbol_value_and_increment(self, t):
+        # todo: need to find less hacky way to parse HQL structure types
+        if "<" == t.value:
+            t.type = "LT"
+            self.lexer.lt_open += 1
+        elif ">" == t.value and not self.lexer.check:
+            t.type = "RT"
+            self.lexer.lt_open -= 1
+        return t
+
+    def after_columns_tokens(self, t):
+        t.type = tok.after_columns_tokens.get(t.value.upper(), t.type)
+        if t.type != "ID":
+            self.lexer.after_columns = True
+        elif self.lexer.columns_def:
+            t.type = tok.columns_defenition.get(t.value.upper(), t.type)
+        return t
+
+    def process_body_tokens(self, t):
+        if self.lexer.last_token == "RP" or self.lexer.after_columns:
+            t = self.after_columns_tokens(t)
+        elif self.lexer.columns_def:
+            t.type = tok.columns_defenition.get(t.value.upper(), t.type)
+        elif self.lexer.sequence:
+            t.type = tok.sequence_reserved.get(t.value.upper(), "ID")
+        return t
+
     def t_STRING(self, t):
         r"\'[a-zA-Z_,0-9:><\=\-\+\~\%$'\!(){}\[\]\/\\\"]*\'\B"
         t.type = "STRING"
@@ -92,44 +52,29 @@ class DDLParser(Parser, HQL):
 
     def t_ID(self, t):
         r"[a-zA-Z_,0-9:><\/\=\-\+\~\%$'\()!{}\[\]\"]+"
-        t.type = "ID"
-        if t.value == ")":
-            t.type = "RP"
-        elif t.value == "(":
-            t.type = "LP"
-            if not self.lexer.after_columns:
-                self.lexer.columns_def = True
-        # todo: need to find less hacky way to parse HQL structure types
-        elif "<" == t.value and not self.lexer.check:
-            t.type = "LT"
-            self.lexer.lt_open += 1
-        elif ">" == t.value and not self.lexer.check:
-            t.type = "RT"
-            self.lexer.lt_open -= 1
-        else:
-            if not self.lexer.is_table:
-                # if is_table mean wi already met INDEX or TABLE statement and the defenition already done and this is a string
-                t.type = self.defenition_statements.get(
-                    t.value.upper(), t.type
-                )  # Check for reserved word
-            t.type = self.common_statements.get(t.value.upper(), t.type)
-
-        if self.lexer.last_token == "RP" or self.lexer.after_columns:
-            t.type = self.after_columns_tokens.get(t.value.upper(), t.type)
-            if t.type != "ID":
-                self.lexer.after_columns = True
-            elif self.lexer.columns_def:
-                t.type = self.columns_defenition.get(t.value.upper(), t.type)
-        elif self.lexer.columns_def:
-            t.type = self.columns_defenition.get(t.value.upper(), t.type)
-        elif self.lexer.sequence:
-            t.type = self.sequence_reserved.get(t.value.upper(), "ID")
+        t.type = tok.symbol_tokens.get(t.value, "ID")
+        if t.type == "LP" and not self.lexer.after_columns:
+            self.lexer.columns_def = True
+            return t
+        elif not self.lexer.check and t.value in tok.symbol_tokens_no_check:
+            return self.get_tag_symbol_value_and_increment(t)
         elif "ARRAY" in t.value:
             t.type = "ARRAY"
-        if t.type == "TABLE" or t.type == "INDEX":
-            self.lexer.is_table = True
-        elif t.type == "SEQUENCE" and self.lexer.is_table:
-            t.type = "ID"
+            return t
+        else:
+            if not self.lexer.is_table:
+                # if is_table mean wi already met INDEX or TABLE statement and
+                # the defenition already done and this is a string
+                t.type = tok.defenition_statements.get(
+                    t.value.upper(), t.type
+                )  # Check for reserved word
+                if t.type == "TABLE" or t.type == "INDEX":
+                    self.lexer.is_table = True
+            t.type = tok.common_statements.get(t.value.upper(), t.type)
+
+        # get tokens from other token dicts
+        t = self.process_body_tokens(t)
+
         if t.type == "SEQUENCE":
             self.lexer.sequence = True
         if t.type == "COMMA" and self.lexer.lt_open:
@@ -177,32 +122,31 @@ class DDLParser(Parser, HQL):
 
     def p_multiple_column(self, p):
         """multiple_column : column
-         | multiple_column COMMA 
-         | multiple_column column
+        | multiple_column COMMA
+        | multiple_column column
         """
         p_list = list(p)
         if isinstance(p[1], dict):
             p[0] = [p[1]]
         else:
             p[0] = p[1]
-            if p_list[-1] != ',':
+            if p_list[-1] != ",":
                 p[0].append(p_list[-1])
-    
-    
+
     def p_id_equals(self, p):
         """id_equals : ID ID ID
         | id_equals COMMA
-        | id_equals COMMA ID ID ID 
+        | id_equals COMMA ID ID ID
         """
         p_list = list(p)
-        if '=' == p_list[-2]:
+        if "=" == p_list[-2]:
             property = {p_list[-3]: p_list[-1]}
             if not isinstance(p[1], list):
                 p[0] = [property]
             else:
                 p[0] = p[1]
                 p[0].append(property)
-        
+
     def p_expression_type_as(self, p):
         """expr : type_name ID LP pid RP
         | type_name ID LP multiple_column RP
@@ -215,18 +159,17 @@ class DDLParser(Parser, HQL):
         if p[0]["base_type"] == "ENUM":
             p[0]["properties"]["values"] = p_list[4]
         elif p[0]["base_type"] == "OBJECT":
-            if 'type' in p_list[4][0]:
+            if "type" in p_list[4][0]:
                 p[0]["properties"]["attributes"] = p_list[4]
         else:
             if isinstance(p_list[-2], list):
                 for item in p_list[-2]:
                     p[0]["properties"].update(item)
 
-        
     def p_type_name(self, p):
         """type_name : type_create ID AS
-        | type_create ID DOT ID AS 
-        | type_create ID DOT ID 
+        | type_create ID DOT ID AS
+        | type_create ID DOT ID
         | type_create ID
         """
         p_list = list(p)
@@ -237,7 +180,7 @@ class DDLParser(Parser, HQL):
         else:
             p[0]["schema"] = p[2]
             p[0]["type_name"] = p_list[4]
-    
+
     def p_type_create(self, p):
         """type_create : CREATE TYPE
         | CREATE OR REPLACE TYPE
@@ -266,14 +209,15 @@ class DDLParser(Parser, HQL):
         p[0]["type_name"] = p_list[-2]
 
     def p_expression_index(self, p):
-        """expr : index_table_name LP pid RP"""
+        """expr : index_table_name LP index_pid RP"""
         p_list = remove_par(list(p))
         p[0] = p[1]
 
-        if not "columns" in p[0]:
-            p[0]["columns"] = p_list[-1]
-        else:
-            p[0]["columns"].append(p_list[-1])
+        for item in ["detailed_columns", "columns"]:
+            if item not in p[0]:
+                p[0][item] = p_list[-1][item]
+            else:
+                p[0][item].extend(p_list[-1][item])
 
     def p_index_table_name(self, p):
         """index_table_name : create_index ON ID
@@ -293,8 +237,13 @@ class DDLParser(Parser, HQL):
         """create_index : CREATE INDEX ID
         | CREATE UNIQUE INDEX ID
         | create_index ON ID
+        | CREATE CLUSTERED INDEX ID
         """
         p_list = list(p)
+        if "CLUSTERED" in p_list:
+            clustered = True
+        else:
+            clustered = False
         if isinstance(p[1], dict):
             p[0] = p[1]
         else:
@@ -302,6 +251,7 @@ class DDLParser(Parser, HQL):
                 "schema": None,
                 "index_name": p_list[-1],
                 "unique": "UNIQUE" in p_list,
+                "clustered": clustered,
             }
 
     def p_expression_table(self, p):
@@ -314,6 +264,10 @@ class DDLParser(Parser, HQL):
         | expr COMMA foreign
         | expr COMMA pkey
         | expr COMMA uniq
+        | expr COMMA constraint uniq
+        | expr COMMA constraint pkey
+        | expr COMMA constraint foreign ref
+        | expr COMMA foreign ref
         | expr RP
         """
         p[0] = p[1]
@@ -328,10 +282,64 @@ class DDLParser(Parser, HQL):
                         check = {"constraint_name": None, "statement": check}
                 else:
                     check = p_list[-1]["check"]
+                    p[0] = self.set_constraint(
+                        p[0], "checks", check, check["constraint_name"]
+                    )
                 p[0]["checks"].append(check)
             else:
                 p[0].update(p_list[-1])
 
+        if isinstance(p_list[-1], dict):
+            if "constraint" in p_list[-2]:
+                if p_list[-1].get("unique_statement"):
+                    p[0] = self.set_constraint(
+                        p[0],
+                        "uniques",
+                        {"columns": p_list[-1]["unique_statement"]},
+                        p_list[-2]["constraint"]["name"],
+                    )
+                else:
+                    print(p_list)
+                    p[0] = self.set_constraint(
+                        p[0],
+                        "primary_keys",
+                        {"columns": p_list[-1]["primary_key"]},
+                        p_list[-2]["constraint"]["name"],
+                    )
+
+            elif p_list[-1].get("references"):
+                p[0] = self.add_ref_information_to_table(p, p_list)
+
+    def add_ref_information_to_table(self, p, p_list):
+        if len(p_list) > 4 and "constraint" in p_list[3]:
+            p[0] = self.set_constraint(
+                p[0],
+                "references",
+                p_list[-1]["references"],
+                p_list[3]["constraint"]["name"],
+            )
+        elif isinstance(p_list[-2], list):
+            if "ref_columns" not in p[0]:
+                p[0]["ref_columns"] = []
+
+            for num, column in enumerate(p_list[-2]):
+                ref = deepcopy(p_list[-1]["references"])
+                ref["column"] = ref["columns"][num]
+                del ref["columns"]
+                ref["name"] = column
+                p[0]["ref_columns"].append(ref)
+        return p[0]
+
+    @staticmethod
+    def set_constraint(target_dict, _type, constraint, constraint_name):
+        if not target_dict.get("constraints"):
+            target_dict["constraints"] = {}
+        if not target_dict["constraints"].get(_type):
+            target_dict["constraints"][_type] = []
+        constraint.update({"constraint_name": constraint_name})
+        target_dict["constraints"][_type].append(constraint)
+        return target_dict
+
     def p_expression_like_table(self, p):
         """expr : table_name LIKE ID
         | table_name LIKE ID DOT ID
@@ -455,7 +463,9 @@ class DDLParser(Parser, HQL):
         else:
             size = None
             p[0] = {"name": p[1], "type": type_str, "size": size}
+
         p_list = remove_par(list(p))
+
         if "[]" == p_list[-1]:
             p[0]["type"] = p[0]["type"] + "[]"
         elif "ARRAY" in p_list[-1]:
@@ -463,25 +473,35 @@ class DDLParser(Parser, HQL):
             append = "[]" if not arr_split[-1] else arr_split[-1]
             p[0]["type"] = p[0]["type"] + append
         elif isinstance(p_list[-1], list):
-            if len(p_list) == 4:
-                p[0]["type"] = f"{p[2]} {p[3][0]}"
-            elif p[0]["type"]:
-                if len(p[0]["type"]) == 1 and isinstance(p[0]["type"], list):
-                    p[0]["type"] = p[0]["type"][0]
-                p[0]["type"] = f'{p[0]["type"]} {p_list[-1][0]}'
-            else:
-                p[0]["type"] = p_list[-1][0]
+            p[0] = self.get_complex_type(p, p_list)
         else:
             match = re.match(r"[0-9]+", p_list[2])
-            if bool(match):
-                size = int(p_list[2])
+            if bool(match) or p_list[2] == "max":
+                if p_list[2].isnumeric():
+                    size = int(p_list[2])
+                else:
+                    size = p_list[2]
                 if len(p_list) == 3:
                     p[0]["size"] = size
                 else:
                     p[0]["size"] = (int(p_list[2]), int(p_list[4]))
+            elif isinstance(p_list[-1], str) and p_list[-1] not in p[0]["type"]:
+                p[0]["type"] += f" {p_list[-1]}"
+
+    @staticmethod
+    def get_complex_type(p, p_list):
+        if len(p_list) == 4:
+            p[0]["type"] = f"{p[2]} {p[3][0]}"
+        elif p[0]["type"]:
+            if len(p[0]["type"]) == 1 and isinstance(p[0]["type"], list):
+                p[0]["type"] = p[0]["type"][0]
+            p[0]["type"] = f'{p[0]["type"]} {p_list[-1][0]}'
+        else:
+            p[0]["type"] = p_list[-1][0]
+        return p[0]
 
     def extract_references(self, p_list):
-        ref_index = p_list.index(_ref)
+        ref_index = p_list.index("REFERENCES")
         ref = {
             "table": None,
             "columns": [None],
@@ -490,7 +510,7 @@ class DDLParser(Parser, HQL):
             "on_update": None,
             "deferrable_initially": None,
         }
-        if not "." in p_list[ref_index:]:
+        if "." not in p_list[ref_index:]:
             ref.update({"table": p_list[ref_index + 1]})
             if not len(p_list) == 3:
                 ref.update({"columns": p_list[-1]})
@@ -518,11 +538,16 @@ class DDLParser(Parser, HQL):
     def p_def(self, p):
         """def : DEFAULT ID
         | DEFAULT STRING
+        | DEFAULT LP pid RP
         | def ID
         | def LP RP
         """
         p_list = list(p)
-        default = p[2]
+        if len(p_list) == 5 and isinstance(p[3], list):
+            default = p[3][0]
+        else:
+            default = p[2]
+
         if default.isnumeric():
             default = int(default)
         if isinstance(p[1], dict):
@@ -534,12 +559,7 @@ class DDLParser(Parser, HQL):
                         p[0]["default"] += f"{i}"
                     else:
                         p[0]["default"] += f" {i}"
-                    p[0]["default"] = (
-                        p[0]["default"]
-                        .replace('"', "")
-                        .replace("'", "")
-                        .replace("\\'", "'")
-                    )
+                    p[0]["default"] = p[0]["default"].replace("))", ")")
         else:
             p[0] = {"default": default}
 
@@ -551,6 +571,8 @@ class DDLParser(Parser, HQL):
         | defcolumn check_ex
         | defcolumn def
         | defcolumn ref
+        | defcolumn foreign ref
+        | defcolumn encrypt
         """
         pk = False
         nullable = True
@@ -611,7 +633,7 @@ class DDLParser(Parser, HQL):
         """
 
         p_list = list(p)
-        con_ind = p_list.index(_cons)
+        con_ind = p_list.index("CONSTRAINT")
         name = p_list[con_ind + 1]
         p[0] = {"constraint": {"name": name}}
 
@@ -633,16 +655,66 @@ class DDLParser(Parser, HQL):
     def p_expression_alter(self, p):
         """expr : alter_foreign ref
         | alter_check
+        | alter_unique
+        | alter_default
         """
         p[0] = p[1]
         if len(p) == 3:
             p[0].update(p[2])
 
+    def p_alter_unique(self, p):
+        """alter_unique : alt_table UNIQUE LP pid RP
+        | alt_table constraint UNIQUE LP pid RP
+        """
+
+        p_list = remove_par(list(p))
+        p[0] = p[1]
+        p[0]["unique"] = {"constraint_name": None, "columns": p_list[-1]}
+        if "constraint" in p[2]:
+            p[0]["unique"]["constraint_name"] = p[2]["constraint"]["name"]
+
+    def p_alter_default(self, p):
+        """alter_default : alt_table ID ID
+        | alt_table constraint ID ID
+        | alt_table ID STRING
+        | alt_table constraint ID STRING
+        | alter_default ID
+        | alter_default FOR pid
+        """
+
+        p_list = remove_par(list(p))
+        p[0] = p[1]
+
+        if "FOR" in p_list:
+            column = p_list[-1]
+            value = None
+        elif p[0].get("default") and "value" in p[0]["default"]:
+            value = p[0]["default"]["value"] + " " + p_list[-1]
+            column = None
+        else:
+            value = p_list[-1]
+            column = None
+        if "default" not in p[0]:
+
+            p[0]["default"] = {
+                "constraint_name": None,
+                "columns": column,
+                "value": value,
+            }
+        else:
+            p[0]["default"].update(
+                {
+                    "columns": p[0]["default"].get("column") or column,
+                    "value": value or p[0]["default"].get("value"),
+                }
+            )
+        if "constraint" in p[2]:
+            p[0]["default"]["constraint_name"] = p[2]["constraint"]["name"]
+
     def p_alter_check(self, p):
         """alter_check : alt_table check_st
         | alt_table constraint check_st
         """
-
         p_list = remove_par(list(p))
         p[0] = p[1]
         if isinstance(p[1], dict):
@@ -667,16 +739,45 @@ class DDLParser(Parser, HQL):
     def p_pid(self, p):
         """pid :  ID
         | STRING
+        | STRING LP RP
+        | ID LP RP
         | pid COMMA ID
         | pid COMMA STRING
         """
         p_list = list(p)
-        if not isinstance(p_list[1], list):
+        if len(p_list) == 4 and isinstance(p[1], str):
+            p[0] = ["".join(p[1:])]
+        elif not isinstance(p_list[1], list):
             p[0] = [p_list[1]]
         else:
             p[0] = p_list[1]
             p[0].append(p_list[-1])
 
+    def p_index_pid(self, p):
+        """index_pid :  ID
+        | index_pid ID
+        | index_pid COMMA index_pid
+        """
+        p_list = list(p)
+        if len(p_list) == 2:
+            detailed_column = {"name": p_list[1], "order": "ASC", "nulls": "LAST"}
+            column = p_list[1]
+            p[0] = {"detailed_columns": [detailed_column], "columns": [column]}
+        else:
+            p[0] = p[1]
+            if len(p) == 3:
+                if p_list[-1] in ["DESC", "ASC"]:
+                    p[0]["detailed_columns"][0]["order"] = p_list[-1]
+                else:
+                    p[0]["detailed_columns"][0]["nulls"] = p_list[-1]
+
+                column = p_list[2]
+            elif isinstance(p_list[-1], dict):
+                for i in p_list[-1]["columns"]:
+                    p[0]["columns"].append(i)
+                for i in p_list[-1]["detailed_columns"]:
+                    p[0]["detailed_columns"].append(i)
+
     def p_alter_foreign(self, p):
         """alter_foreign : alt_table foreign
         | alt_table constraint foreign
@@ -714,12 +815,12 @@ class DDLParser(Parser, HQL):
 
     def p_foreign(self, p):
         # todo: need to redone id lists
-        """foreign : FOREIGN KEY LP pid RP"""
+        """foreign : FOREIGN KEY LP pid RP
+        | FOREIGN KEY"""
         p_list = remove_par(list(p))
-        key_index = p_list.index("KEY")
-        columns = p_list[-1]
-
-        p[0] = columns
+        if len(p_list) == 4:
+            columns = p_list[-1]
+            p[0] = columns
 
     def p_ref(self, p):
         """ref : REFERENCES ID DOT ID
@@ -758,33 +859,25 @@ class DDLParser(Parser, HQL):
         p[0] = p[1]
 
     def p_uniq(self, p):
-        """uniq : UNIQUE LP ID
-        | uniq COMMA
-        | uniq ID
-        | uniq RP
-
-        """
-        p_list = list(p)
-        if isinstance(p[1], dict):
-            p[0] = p[1]
-            p[0]["unique_statement"].append(p_list[-1])
-        else:
-            p[0] = {"unique_statement": [x for x in p[2:] if x != ","]}
+        """uniq : UNIQUE LP pid RP"""
+        p_list = remove_par(list(p))
+        p[0] = {"unique_statement": p_list[-1]}
 
     def p_pkey(self, p):
         """pkey : PRIMARY KEY LP pid RP"""
         p_list = remove_par(list(p))
         p[0] = {"primary_key": p_list[-1]}
 
+    def p_tablespace(self, p):
+        """tablespace : TABLESPACE ID"""
+        # Initial 5m Next 5m Maxextents Unlimited
+        p[0] = p[2]
 
-def remove_par(p_list):
-    if "(" in p_list:
-        p_in = p_list.index("(")
-        p_list.pop(p_in)
-    if ")" in p_list:
-        p_in = p_list.index(")")
-        p_list.pop(p_in)
-    return p_list
+    def p_expr_tablespace(self, p):
+        """expr : expr tablespace"""
+        p_list = list(p)
+        p[0] = p[1]
+        p[0]["tablespace"] = p_list[-1]
 
 
 def parse_from_file(file_path: str, **kwargs) -> List[Dict]:
