@@ -1,7 +1,8 @@
 from typing import Dict, List
 
-
 hql_clean_up_list = ["deferrable_initially"]
+
+
 sql_clean_up_list = [
     "external",
     "external",
@@ -28,12 +29,25 @@ def add_additional_hql_keys(table_data: Dict) -> Dict:
     return table_data
 
 
+def add_additional_oracle_keys(table_data: Dict) -> Dict:
+    table_data.update(
+        {
+            "constraints": {"uniques": None, "checks": None, "references": None},
+            "storage": None,
+        }
+    )
+    return table_data
+
+
+def add_additional_oracle_keys_in_column(table_data: Dict) -> Dict:
+    table_data.update({"encrypt": None})
+    return table_data
+
+
 def add_additional_mssql_keys(table_data: Dict) -> Dict:
     table_data.update(
         {
-            "constraints": {"uniques": None, 
-                            "checks": None, 
-                            "references": None},
+            "constraints": {"uniques": None, "checks": None, "references": None},
         }
     )
     return table_data
@@ -49,8 +63,10 @@ def clean_up_output(table_data: Dict, key_list: List[str]) -> Dict:
 def populate_dialects_table_data(output_mode: str, table_data: Dict) -> Dict:
     if output_mode == "hql":
         table_data = add_additional_hql_keys(table_data)
-    elif output_mode in ["mssql", "mysql", "oracle"]:
+    elif output_mode in ["mssql", "mysql"]:
         table_data = add_additional_mssql_keys(table_data)
+    elif output_mode == "oracle":
+        table_data = add_additional_oracle_keys(table_data)
     return table_data
 
 
@@ -62,5 +78,7 @@ def dialects_clean_up(output_mode: str, table_data: Dict) -> Dict:
         # todo: need to figure out how workaround it normally
         if "_ddl_parser_comma_only_str" == table_data["fields_terminated_by"]:
             table_data["fields_terminated_by"] = ","
-
+    if output_mode == "oracle":
+        for column in table_data["columns"]:
+            column = add_additional_oracle_keys_in_column(column)
     return table_data
